"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mkdirp_1 = __importDefault(require("mkdirp"));
const rimraf_1 = __importDefault(require("rimraf"));
const webpack_1 = __importDefault(require("webpack"));
const set_1 = __importDefault(require("lodash/set"));
const util_1 = require("util");
const loadEntries_1 = __importDefault(require("./loadEntries"));
const patchEntries_1 = __importDefault(require("./patchEntries"));
const addHotMiddleware_1 = __importDefault(require("./addHotMiddleware"));
const Entry_1 = require("./Entry");
const rimrafAsync = util_1.promisify(rimraf_1.default);
class EntryList {
    constructor(options) {
        this.options = options;
        this.entries = [];
    }
    get publicPath() {
        var _a;
        return (_a = this._publicPath) !== null && _a !== void 0 ? _a : "/";
    }
    static build(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const instance = new EntryList(options);
            const entriesConfig = yield loadEntries_1.default(options.packsDirectory, [], options.acceptFile);
            const { compiler, config } = yield instance.setup({ entriesConfig });
            return { compiler, config, instance };
        });
    }
    create(url, mode) {
        const entry = this.entries.find((entry) => entry.test(url));
        if (!entry) {
            return Promise.reject(new Error(`Can't find entry`));
        }
        return entry.create(mode);
    }
    setup(opt) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const entryMap = patchEntries_1.default(opt.entriesConfig, this.options.SRC);
            const { SRC, packsDirectory, code, host } = this.options;
            const config = Object.assign(Object.assign({}, this.options.config), { entry: addHotMiddleware_1.default(entryMap, host, this.options.hotMiddlewareOptions), plugins: (this.options.config.plugins || []).concat([
                    new webpack_1.default.HotModuleReplacementPlugin(),
                ]) });
            const publicPath = (_b = (_a = this.options.config) === null || _a === void 0 ? void 0 : _a.output) === null || _b === void 0 ? void 0 : _b.publicPath;
            this._publicPath = publicPath;
            if (publicPath) {
                set_1.default(config, "output.publicPath", host + publicPath);
            }
            Object.keys(entryMap).forEach((key) => {
                const entry = new Entry_1.Entry({
                    SRC,
                    name: key,
                    publicPath: this.publicPath,
                    packsDirectory,
                    code,
                });
                this.entries.push(entry);
            });
            yield this.clearFolder();
            yield this.createFolder();
            const deferEntries = this.entries.reduce((memo, entry) => {
                memo.push(entry.create("empty"));
                memo.push(entry.createHot());
                return memo;
            }, []);
            yield Promise.all(deferEntries);
            const compiler = webpack_1.default(config);
            const iFS = compiler.inputFileSystem;
            const writeFileAsyncNotify = (...args) => new Promise((resolve, reject) => iFS.fileSystem.writeFile(...args, (err, data) => err ? reject(err) : resolve(data)));
            this.entries.forEach((entry) => (entry.writeFileAsyncNotify = writeFileAsyncNotify));
            return { compiler, config };
        });
    }
    clearFolder() {
        // clear virtual folder
        return rimrafAsync(this.options.SRC);
    }
    createFolder() {
        // create virtual folder
        return mkdirp_1.default(this.options.SRC);
    }
}
exports.default = EntryList;
//# sourceMappingURL=EntryList.js.map